unit TestX.UClass.TAritmeticaBasica;

interface
uses
  System.SysUtils,
  DUnitX.TestFramework,
  UClass.TAritmeticaBasica,
  DUnitX.Types;

type

  [TestFixture]
  [Category('Ejemplos Blog')]
  TestXTAritmeticaBasica = class(TObject)
  strict private
    FAritmeticaBasica: TAritmeticaBasica;
    FFactorizacion:TIntegerList;

  private
    FContador:Integer;
  public
    constructor Create;
    [Setup]
    procedure Setup;
    [TearDown]
    procedure TearDown;

    [Test]
    [Category('NumeroPrimo')]
    [TestCase('Test_0', '0, 0')]
    [TestCase('Test_1', '1, 1')]
    [TestCase('Test_2', '2, 1')]
    [TestCase('Test_3', '3, 1')]
    [TestCase('Test_4', '4, 0')]
    [TestCase('Test_20','20,0')]
    [TestCase('Test_11','11,1')]
    [TestCase('Test_18','18,0')]
    [TestCase('Test_17','17,1')]
    procedure TestEsNumeroPrimo(Value1: Int64; Result:integer);

    [Test]
    [TestCase('Test_Factorizacion_0', '0,0')]
    [TestCase('Test_Factorizacion_1', '1,0')]
    [TestCase('Test_Factorizacion_2', '2,1')]
    [TestCase('Test_Factorizacion_4', '4, 2,2,2')]
    [TestCase('Test_Factorizacion_10','10,2,2,5')]
    [TestCase('Test_Factorizacion_18','18,3,2,3,3')]
    procedure TestFactorizacion(Value1:Int64; Count:integer; Result1, Result2, Result3:integer);

    [Test]
    [TestCase('Test_MinimoComunMultiplo_1_1','1,1,1')]
    [TestCase('Test_MinimoComunMultiplo_1_2','1,2,2')]
    [TestCase('Test_MinimoComunMultiplo_20_3','20,3,60')]
    [TestCase('Test_MinimoComunMultiplo_40_9','40,9,360')]
    [TestCase('Test_MinimoComunMultiplo_50_4','50,4,100')]
    [TestCase('Test_MinimoComunMultiplo_25_4','25,4,100')]
    [TestCase('Test_MinimoComunMultiplo_111_5','111,5,555')]
    [TestCase('Test_MinimoComunMultiplo_1000_40','1000,40,1000')]
    procedure TestMinimoComunMultiplo(ANumero1, ANumero2:Int64; AResult:Int64);

    [Test]
    [TestCase('Test_MaximoComunDivisor_1_1','1,1,1')]
    [TestCase('Test_MaximoComunDivisor_1_2','1,2,1')]
    [TestCase('Test_MaximoComunDivisor_20_8','20,8,4')]
    [TestCase('Test_MaximoComunDivisor_45_12','45,12,3')]
    [TestCase('Test_MaximoComunDivisor_90_20','90,20,10')]
    [TestCase('Test_MaximoComunDivisor_200_60','200,60,20')]
    procedure TestMaximoComunDivisor(ANumero1, ANumero2:Int64; AResult:Int64);

  // Los ponenmos private para ignorarlos
  private
    [Test]
    [RepeatTest(10)]
    [Ignore]
    procedure RepetirTest;

    [Test]
    [Ignore]
    [TestCase('Test_GenerarLeaksMemoria', '1')]
    [TestCase('Test_GenerarLeaksMemoria', '2')]
    [TestCase('Test_GenerarLeaksMemoria', '3')]
    procedure TestGeneraLeaksDeMemoria(AValue:integer);

    [Test]
    [Ignore]
    [MaxTime(2000)]
    procedure MetodoTarda5sg;
  end;


implementation

procedure TestXTAritmeticaBasica.MetodoTarda5sg;
var
  i:Integer;
begin
  for i := 0 to 4 do
    Sleep(1000);
  Assert.Pass('Finalizado; 5 segundos.');
end;

procedure TestXTAritmeticaBasica.TestGeneraLeaksDeMemoria(AValue: integer);
var
  i:Integer;
begin
  i := FAritmeticaBasica.GeneraLeaksDeMemoria(AValue);
  Assert.AreEqual(i, 1);
end;

procedure TestXTAritmeticaBasica.TestMaximoComunDivisor(ANumero1, ANumero2:Int64; AResult:Int64);
var
  ReturnValue: Int64;
begin
  ReturnValue := FAritmeticaBasica.MaximoComunDivisor(ANumero1, ANumero2);
  Assert.AreEqual(ReturnValue, AResult, 'El resultado del MCD debera ser: ' + IntToStr(ReturnValue));
end;

procedure TestXTAritmeticaBasica.TestMinimoComunMultiplo;
var
  ReturnValue: Int64;
begin
  ReturnValue := FAritmeticaBasica.MinimoComunMultiplo(ANumero1, ANumero2);
  Assert.AreEqual(ReturnValue, AResult, 'El resultado del MCM debera ser: ' + IntToStr(ReturnValue));
end;

procedure TestXTAritmeticaBasica.TestEsNumeroPrimo(Value1: Int64; Result: integer);
var
  ResultValue:boolean;
begin
  ResultValue := FAritmeticaBasica.EsNumeroPrimo(Value1);
  Assert.AreEqual(ResultValue, (Result = 1));
end;



procedure TestXTAritmeticaBasica.RepetirTest;
begin
  Inc(FContador);
  Assert.AreNotEqual(FContador, 1, 'Contador no debe ser igual a 1');
end;

constructor TestXTAritmeticaBasica.Create;
begin
  inherited;
  // Ini variales
  FContador := 0;
end;

//procedure TestXTAritmeticaBasica.RaiseEConvertError;
//begin
//
//  // Excepcin genrica
//  //-- raise EArgumentException.Create('Excepcin de tipo genrico');
//  // excepcin del tipo esperado
//  //-- raise EConvertError.Create('Se ha realizado una conversin de tipos incorrecta.');
//
//  FAritmeticaBasica.GeneraEConvertError;
//  //  Assert.WillRaise(FAritmeticaBasica.GeneraEconvertError, EConvertError, 'Tipo de excepcin oncorrecto');
//  // Assert.WillRaise(FAritmeticaBasica.GeneraEconvertError, EConvertError, 'La excepcin generada es del tipo incorrecto.');
//  //--Assert.WillRaise(FAritmeticaBasica.GeneraEArgumentException, EConvertError, 'Tipo de excepcin oncorrecto');
//
//
//
//end;

procedure TestXTAritmeticaBasica.Setup;
begin
  // Crear el objeto que vamos a probar
  FAritmeticaBasica := TAritmeticaBasica.Create;
  FFactorizacion := TIntegerList.Create;
end;

procedure TestXTAritmeticaBasica.TearDown;
begin
  // Liberar el objeto de prueba
  FAritmeticaBasica.Free;
  FAritmeticaBasica := nil;
  FreeAndNil(FFactorizacion);
end;



procedure TestXTAritmeticaBasica.TestFactorizacion(Value1:Int64; Count:integer; Result1, Result2, Result3:integer);
begin
  FAritmeticaBasica.Factorizacion(Value1, FFactorizacion);
  Assert.AreEqual(FFactorizacion.Count, Count,
                  Format('Se esperaban %d factores y se han obtenido %d',[Count, FFactorizacion.Count]));
end;




//==============================================================================
//
//  I N I T I A L I Z A T I O N
//
//==============================================================================
initialization
  TDUnitX.RegisterTestFixture(TestXTAritmeticaBasica);

end.
